#version-warning {
  top: 0;
  position: sticky;
  z-index: 60;
  width: 100%;
  height: 2.5rem;
  display: flex;
  column-gap: 0.5rem;
  justify-content: center;
  justify-items: center;
  align-items: center;
  background-color: #eee;
  border-bottom: 2px solid #ae2828;
}

@media (prefers-color-scheme: dark) {
  body:not([data-theme="light"]) #version-warning {
    background-color: black;
  }
}

.breaking-change {
  font-variant: all-small-caps;
  margin-left: 0.4rem;
  color: #f55353;
}

p {
  font-size: 1.1em;
}

html[data-theme="dark"] .mermaid svg {
  background-color: white;
  padding: 1em;
}

html[data-theme="dark"] .mermaid svg p {
  color: black;
}
